/* CNRS/LAAS, 2011
 * Public domain
 */


#include "jointMapParser.hpp"
#include <iostream>
#include <string.h>

using namespace std;

void JointMapParser::readJointMapFile(ori_jointmap* jointMap, std:: string fileName){
  ifstream file(fileName.c_str());
  string line;
  if(!jointMap){
    cout << "The given jointMap structure is NULL" << endl;
    return;
  }
  
  if(file.is_open()){
    unsigned int nb_dofs = 0;
    while (file.good()){
      getline (file,line);
      if(line.length() != 0){
        strcpy (jointMap->joints[nb_dofs], line.c_str());
        ++nb_dofs;
      }
    }
    jointMap->nb_dofs = nb_dofs;
    file.close();
  }else{
    cout << "Unable to open map file : " << fileName << endl;
  }
}

void JointMapParser::readMappingFile(std::map <std::string, std::string, std::less<std::string> > &mapping, std::string fileName, char separator){
  ifstream file (fileName.c_str());
  string line;
  size_t separatorStart, separatorEnd;
  
  if(file.is_open()){
    while (file.good()){
      getline (file,line);
      if(line.length() != 0){
        separatorStart = line.find(separator);
        separatorEnd = line.rfind(separator);
        if (separatorStart != string::npos && separatorEnd != string::npos){
          pair <string, string> jointMap;
          jointMap.first = line.substr(0,separatorStart);
          jointMap.second = line.substr(separatorEnd + 1, line.length() - separatorEnd);
          mapping.insert(jointMap);
        }else{
          cout << "Unable to find separator : '" << separator << "' in : " << line << endl;
        }
      }
    }
    file.close();
  }else{
    cout << "Unable to open map file : " << fileName << endl;
  }
}
