/* CNRS/LAAS, 2011
 * Public domain
 */
#include "jointStateMapper.hpp"
#include <string.h>
#include <iostream>

using namespace std;

JointStateMapper::JointStateMapper (){
  _physical2SimulationMap.clear();
  _physicalMap.nb_dofs = 0;
  _simulationMap.nb_dofs = 0;
}

JointStateMapper::JointStateMapper (const ori_jointmap& physicalMap, const ori_jointmap& simulationMap, const std::map <string, string, less<string> > mapping){
  /* Class init */
  init(physicalMap, simulationMap, mapping);
}

JointStateMapper::JointStateMapper (const ori_jointmap& physicalMap, const ori_jointmap& simulationMap, string mappingFileName){
  /* File reading */
  std::map <string, string, less<string> > mapping;
  JointMapParser::readMappingFile(mapping, mappingFileName, '\t');
  
  /* Class init */
  init(physicalMap, simulationMap, mapping);
}
JointStateMapper::JointStateMapper (const std::string physicalMapFileName, const ori_jointmap& simulationMap, const std::string mappingFileName){
  ori_jointmap physicalMap;
  JointMapParser::readJointMapFile(&physicalMap, physicalMapFileName);
  map <string, string, less<string> > mapping;
  JointMapParser::readMappingFile(mapping, mappingFileName, '\t');

  /* Class init */
  init(physicalMap, simulationMap, mapping);
}

JointStateMapper::JointStateMapper (const ori_jointmap& physicalMap, const std::string simulationMapFileName, const std::string mappingFileName){
  ori_jointmap simulationMap;
  JointMapParser::readJointMapFile(&simulationMap, simulationMapFileName);
  map <string, string, less<string> > mapping;
  JointMapParser::readMappingFile(mapping, mappingFileName, '\t');

  /* Class init */
  init(physicalMap, simulationMap, mapping);
}

JointStateMapper::JointStateMapper (string physicalMapFileName, string simulationMapFileName, string mappingFileName){
  /* File reading */
  ori_jointmap physicalMap, simulationMap;
  JointMapParser::readJointMapFile(&physicalMap, physicalMapFileName);
  JointMapParser::readJointMapFile(&simulationMap, simulationMapFileName);
  map <string, string, less<string> > mapping;
  JointMapParser::readMappingFile(mapping, mappingFileName, '\t');

  /* Class init */
  init(physicalMap, simulationMap, mapping);
}

JointStateMapper::~JointStateMapper (){
  /* Nothing to do */
}


void JointStateMapper::init (const ori_jointmap& physicalMap, const ori_jointmap& simulationMap, const map <string, string, less<string> > mapping){
  copyOriJointMap(physicalMap, &_physicalMap);
  copyOriJointMap(simulationMap, &_simulationMap);

  for(unsigned int i = 0; i < _physicalMap.nb_dofs; i++){
    bool found = false;
    pair<int, int> jntMap;
    jntMap.first = i;
    map <string, string>::const_iterator it;
    it = mapping.find(_physicalMap.joints[i]);
    string simuJntName;
    if(it != mapping.end()){
      simuJntName = it->second;
      for(unsigned int j = 0; j < _simulationMap.nb_dofs; j++){
        if(!strcmp(_simulationMap.joints[j], simuJntName.c_str())){
          found = true;
          jntMap.second = j;
        }
      }
      if(found){
        _physical2SimulationMap.insert(jntMap);
      }else{
        cout << "Can not find mapping joint for physical joint : " << simuJntName << endl;
      }
    }
  }
}


int JointStateMapper::fillSimulationJointState(const ori_jointstate& physicalJs, ori_jointstate &simulationJs){
  if(physicalJs.nb_dofs != _physicalMap.nb_dofs){
    cout << "Error in fillSimulationJointState : Invalid nb_dofs" << endl;
    cout << "Pysical Given : " << physicalJs.nb_dofs << " Saved : " << _physicalMap.nb_dofs << endl;
    return 1;
  }
  for(map <int,int>::iterator iter = _physical2SimulationMap.begin(); iter != _physical2SimulationMap.end(); iter++){
    simulationJs.q[iter->second] = physicalJs.q[iter->first];
  }

  simulationJs.nb_dofs = _simulationMap.nb_dofs;
  return 0;
}
int JointStateMapper::getPhysicalJointId(int jointId){
  if(jointId > _simulationMap.nb_dofs){
    cout << "Error in getPhysicalJointId : Invalid nb_dofs" << endl;
    return -1;
  }
  for(map <int,int>::iterator iter = _physical2SimulationMap.begin(); iter != _physical2SimulationMap.end(); iter++){
    if(iter->second == jointId){
      return iter->first;
    }
  }
  return -1;
}
int JointStateMapper::getPhysicalJointId(string jointName){
  for(int i = 0; i < _simulationMap.nb_dofs; i++){
    if(!strcmp(_simulationMap.joints[i], jointName.c_str())){
      return getPhysicalJointId(i);
    }
  }
  return -1;
}
int JointStateMapper::getSimulationJointId(int jointId){
  if(jointId > _physicalMap.nb_dofs){
    cout << "Error in getSimulationJointId : Invalid nb_dofs" << endl;
    return -1;
  }
  map <int,int>::iterator iter = _physical2SimulationMap.find(jointId);
  if(iter != _physical2SimulationMap.end()){
    return iter->second;
  }
  return -1;
}
int JointStateMapper::getSimulationJointId(string jointName){
  for(int i = 0; i < _physicalMap.nb_dofs; i++){
    if(!strcmp(_physicalMap.joints[i], jointName.c_str())){
      return getSimulationJointId(i);
    }
  }
  return -1;
}

void JointStateMapper::copyOriJointMap(const ori_jointmap& jmIn, ori_jointmap* jmOut){
  jmOut->nb_dofs = jmIn.nb_dofs;
  for(unsigned int i = 0; i < jmOut->nb_dofs; i++){
    strcpy(jmOut->joints[i], jmIn.joints[i]);
  }
}
